import torch
from torch_sparse import SparseTensor
from torch_geometric.nn import GCN2Conv


def test_gcn2_conv():
    x = torch.randn(4, 16)
    x_0 = torch.randn(4, 16)
    edge_index = torch.tensor([[0, 0, 0, 1, 2, 3], [1, 2, 3, 0, 0, 0]])
    row, col = edge_index
    value = torch.rand(row.size(0))
    adj2 = SparseTensor(row=row, col=col, value=value, sparse_sizes=(4, 4))
    adj1 = adj2.set_value(None)

    conv = GCN2Conv(16, alpha=0.2)
    assert conv.__repr__() == 'GCN2Conv(16, alpha=0.2, beta=1.0)'
    out1 = conv(x, x_0, edge_index)
    assert out1.size() == (4, 16)
    assert torch.allclose(conv(x, x_0, adj1.t()), out1, atol=1e-6)
    out2 = conv(x, x_0, edge_index, value)
    assert out2.size() == (4, 16)
    assert torch.allclose(conv(x, x_0, adj2.t()), out2, atol=1e-6)

    t = '(Tensor, Tensor, Tensor, OptTensor) -> Tensor'
    jit = torch.jit.script(conv.jittable(t))
    assert jit(x, x_0, edge_index).tolist() == out1.tolist()
    assert jit(x, x_0, edge_index, value).tolist() == out2.tolist()

    t = '(Tensor, Tensor, SparseTensor, OptTensor) -> Tensor'
    jit = torch.jit.script(conv.jittable(t))
    assert torch.allclose(conv(x, x_0, adj1.t()), out1, atol=1e-6)
    assert torch.allclose(conv(x, x_0, adj2.t()), out2, atol=1e-6)

    conv.cached = True
    conv(x, x_0, edge_index)
    assert conv(x, x_0, edge_index).tolist() == out1.tolist()
    conv(x, x_0, adj1.t())
    assert torch.allclose(conv(x, x_0, adj1.t()), out1, atol=1e-6)
